@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#
+# -*- coding: utf-8 -*-
 # Copyright 2009 Facebook
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -18,36 +18,38 @@
 
 To load a locale and generate a translated string::
 
-    user_locale = locale.get("es_LA")
+    user_locale = tornado.locale.get("es_LA")
     print user_locale.translate("Sign out")
 
-locale.get() returns the closest matching locale, not necessarily the
+`tornado.locale.get()` returns the closest matching locale, not necessarily the
 specific locale you requested. You can support pluralization with
-additional arguments to translate(), e.g.::
+additional arguments to `~Locale.translate()`, e.g.::
 
     people = [...]
     message = user_locale.translate(
         "%(list)s is online", "%(list)s are online", len(people))
     print message % {"list": user_locale.list(people)}
 
-The first string is chosen if len(people) == 1, otherwise the second
+The first string is chosen if ``len(people) == 1``, otherwise the second
 string is chosen.
 
-Applications should call one of load_translations (which uses a simple
-CSV format) or load_gettext_translations (which uses the .mo format
-supported by gettext and related tools).  If neither method is called,
-the locale.translate method will simply return the original string.
+Applications should call one of `load_translations` (which uses a simple
+CSV format) or `load_gettext_translations` (which uses the ``.mo`` format
+supported by `gettext` and related tools).  If neither method is called,
+the `Locale.translate` method will simply return the original string.
 """
 
-from __future__ import absolute_import, division, with_statement
+from __future__ import absolute_import, division, print_function, with_statement
 
 import csv
 import datetime
-import logging
+import numbers
 import os
 import re
 
 from tornado import escape
+from tornado.log import gen_log
+from tornado.util import u
 
 _default_locale = "en_US"
 _translations = {}
@@ -62,15 +64,15 @@ def get(*locale_codes):
     or a loose match for the code (e.g., "en" for "en_US"), we return
     the locale. Otherwise we move to the next code in the list.
 
-    By default we return en_US if no translations are found for any of
+    By default we return ``en_US`` if no translations are found for any of
     the specified locales. You can change the default locale with
-    set_default_locale() below.
+    `set_default_locale()`.
     """
     return Locale.get_closest(*locale_codes)
 
 
 def set_default_locale(code):
-    """Sets the default locale, used in get_closest_locale().
+    """Sets the default locale.
 
     The default locale is assumed to be the language used for all strings
     in the system. The translations loaded from disk are mappings from
@@ -80,34 +82,34 @@ def set_default_locale(code):
     global _default_locale
     global _supported_locales
     _default_locale = code
-    _supported_locales = frozenset(_translations.keys() + [_default_locale])
+    _supported_locales = frozenset(list(_translations.keys()) + [_default_locale])
 
 
 def load_translations(directory):
-    u"""Loads translations from CSV files in a directory.
+    """Loads translations from CSV files in a directory.
 
     Translations are strings with optional Python-style named placeholders
-    (e.g., "My name is %(name)s") and their associated translations.
+    (e.g., ``My name is %(name)s``) and their associated translations.
 
-    The directory should have translation files of the form LOCALE.csv,
-    e.g. es_GT.csv. The CSV files should have two or three columns: string,
+    The directory should have translation files of the form ``LOCALE.csv``,
+    e.g. ``es_GT.csv``. The CSV files should have two or three columns: string,
     translation, and an optional plural indicator. Plural indicators should
     be one of "plural" or "singular". A given string can have both singular
-    and plural forms. For example "%(name)s liked this" may have a
+    and plural forms. For example ``%(name)s liked this`` may have a
     different verb conjugation depending on whether %(name)s is one
     name or a list of names. There should be two rows in the CSV file for
     that string, one with plural indicator "singular", and one "plural".
     For strings with no verbs that would change on translation, simply
     use "unknown" or the empty string (or don't include the column at all).
 
-    The file is read using the csv module in the default "excel" dialect.
+    The file is read using the `csv` module in the default "excel" dialect.
     In this format there should not be spaces after the commas.
 
-    Example translation es_LA.csv:
+    Example translation ``es_LA.csv``::
 
         "I love you","Te amo"
-        "%(name)s liked this","A %(name)s les gust\u00f3 esto","plural"
-        "%(name)s liked this","A %(name)s le gust\u00f3 esto","singular"
+        "%(name)s liked this","A %(name)s les gustó esto","plural"
+        "%(name)s liked this","A %(name)s le gustó esto","singular"
 
     """
     global _translations
@@ -118,7 +120,7 @@ def load_translations(directory):
             continue
         locale, extension = path.split(".")
         if not re.match("[a-z]+(_[A-Z]+)?$", locale):
-            logging.error("Unrecognized locale %r (path: %s)", locale,
+            gen_log.error("Unrecognized locale %r (path: %s)", locale,
                           os.path.join(directory, path))
             continue
         full_path = os.path.join(directory, path)
@@ -128,8 +130,6 @@ def load_translations(directory):
             f = open(full_path, "r", encoding="utf-8")
         except TypeError:
             # python 2: files return byte strings, which are decoded below.
-            # Once we drop python 2.5, this could use io.open instead
-            # on both 2 and 3.
             f = open(full_path, "r")
         _translations[locale] = {}
         for i, row in enumerate(csv.reader(f)):
@@ -142,32 +142,35 @@ def load_translations(directory):
             else:
                 plural = "unknown"
             if plural not in ("plural", "singular", "unknown"):
-                logging.error("Unrecognized plural indicator %r in %s line %d",
+                gen_log.error("Unrecognized plural indicator %r in %s line %d",
                               plural, path, i + 1)
                 continue
             _translations[locale].setdefault(plural, {})[english] = translation
         f.close()
-    _supported_locales = frozenset(_translations.keys() + [_default_locale])
-    logging.debug("Supported locales: %s", sorted(_supported_locales))
+    _supported_locales = frozenset(list(_translations.keys()) + [_default_locale])
+    gen_log.debug("Supported locales: %s", sorted(_supported_locales))
 
 
 def load_gettext_translations(directory, domain):
-    """Loads translations from gettext's locale tree
+    """Loads translations from `gettext`'s locale tree
 
-    Locale tree is similar to system's /usr/share/locale, like:
+    Locale tree is similar to system's ``/usr/share/locale``, like::
 
-    {directory}/{lang}/LC_MESSAGES/{domain}.mo
+        {directory}/{lang}/LC_MESSAGES/{domain}.mo
 
     Three steps are required to have you app translated:
 
-    1. Generate POT translation file
-        xgettext --language=Python --keyword=_:1,2 -d cyclone file1.py file2.html etc
+    1. Generate POT translation file::
 
-    2. Merge against existing POT file:
-        msgmerge old.po cyclone.po > new.po
+        xgettext --language=Python --keyword=_:1,2 -d mydomain file1.py file2.html etc
 
-    3. Compile:
-        msgfmt cyclone.po -o {directory}/pt_BR/LC_MESSAGES/cyclone.mo
+    2. Merge against existing POT file::
+
+        msgmerge old.po mydomain.po > new.po
+
+    3. Compile::
+
+        msgfmt mydomain.po -o {directory}/pt_BR/LC_MESSAGES/mydomain.mo
     """
     import gettext
     global _translations
@@ -183,12 +186,12 @@ def load_gettext_translations(directory, domain):
             os.stat(os.path.join(directory, lang, "LC_MESSAGES", domain + ".mo"))
             _translations[lang] = gettext.translation(domain, directory,
                                                       languages=[lang])
-        except Exception, e:
-            logging.error("Cannot load translation for '%s': %s", lang, str(e))
+        except Exception as e:
+            gen_log.error("Cannot load translation for '%s': %s", lang, str(e))
             continue
-    _supported_locales = frozenset(_translations.keys() + [_default_locale])
+    _supported_locales = frozenset(list(_translations.keys()) + [_default_locale])
     _use_gettext = True
-    logging.debug("Supported locales: %s", sorted(_supported_locales))
+    gen_log.debug("Supported locales: %s", sorted(_supported_locales))
 
 
 def get_supported_locales():
@@ -242,7 +245,7 @@ class Locale(object):
 
     def __init__(self, code, translations):
         self.code = code
-        self.name = LOCALE_NAMES.get(code, {}).get("name", u"Unknown")
+        self.name = LOCALE_NAMES.get(code, {}).get("name", u("Unknown"))
         self.rtl = False
         for prefix in ["fa", "ar", "he"]:
             if self.code.startswith(prefix):
@@ -263,9 +266,10 @@ class Locale(object):
     def translate(self, message, plural_message=None, count=None):
         """Returns the translation for the given message for this locale.
 
-        If plural_message is given, you must also provide count. We return
-        plural_message when count != 1, and we return the singular form
-        for the given message when count == 1.
+        If ``plural_message`` is given, you must also provide
+        ``count``. We return ``plural_message`` when ``count != 1``,
+        and we return the singular form for the given message when
+        ``count == 1``.
         """
         raise NotImplementedError()
 
@@ -274,17 +278,17 @@ class Locale(object):
         """Formats the given date (which should be GMT).
 
         By default, we return a relative time (e.g., "2 minutes ago"). You
-        can return an absolute date string with relative=False.
+        can return an absolute date string with ``relative=False``.
 
         You can force a full format date ("July 10, 1980") with
-        full_format=True.
+        ``full_format=True``.
 
         This method is primarily intended for dates in the past.
         For dates in the future, we fall back to full format.
         """
         if self.code.startswith("ru"):
             relative = False
-        if type(date) in (int, long, float):
+        if isinstance(date, numbers.Real):
             date = datetime.datetime.utcfromtimestamp(date)
         now = datetime.datetime.utcnow()
         if date > now:
@@ -323,26 +327,26 @@ class Locale(object):
             if days == 0:
                 format = _("%(time)s")
             elif days == 1 and local_date.day == local_yesterday.day and \
-                 relative:
+                    relative:
                 format = _("yesterday") if shorter else \
-                         _("yesterday at %(time)s")
+                    _("yesterday at %(time)s")
             elif days < 5:
                 format = _("%(weekday)s") if shorter else \
-                         _("%(weekday)s at %(time)s")
+                    _("%(weekday)s at %(time)s")
             elif days < 334:  # 11mo, since confusing for same month last year
                 format = _("%(month_name)s %(day)s") if shorter else \
-                         _("%(month_name)s %(day)s at %(time)s")
+                    _("%(month_name)s %(day)s at %(time)s")
 
         if format is None:
             format = _("%(month_name)s %(day)s, %(year)s") if shorter else \
-                     _("%(month_name)s %(day)s, %(year)s at %(time)s")
+                _("%(month_name)s %(day)s, %(year)s at %(time)s")
 
         tfhour_clock = self.code not in ("en", "en_US", "zh_CN")
         if tfhour_clock:
             str_time = "%d:%02d" % (local_date.hour, local_date.minute)
         elif self.code == "zh_CN":
             str_time = "%s%d:%02d" % (
-                (u'\u4e0a\u5348', u'\u4e0b\u5348')[local_date.hour >= 12],
+                (u('\u4e0a\u5348'), u('\u4e0b\u5348'))[local_date.hour >= 12],
                 local_date.hour % 12 or 12, local_date.minute)
         else:
             str_time = "%d:%02d %s" % (
@@ -361,7 +365,7 @@ class Locale(object):
         """Formats the given date as a day of week.
 
         Example: "Monday, January 22". You can remove the day of week with
-        dow=False.
+        ``dow=False``.
         """
         local_date = date - datetime.timedelta(minutes=gmt_offset)
         _ = self.translate
@@ -388,7 +392,7 @@ class Locale(object):
             return ""
         if len(parts) == 1:
             return parts[0]
-        comma = u' \u0648 ' if self.code.startswith("fa") else u", "
+        comma = u(' \u0648 ') if self.code.startswith("fa") else u(", ")
         return _("%(commas)s and %(last)s") % {
             "commas": comma.join(parts[:-1]),
             "last": parts[len(parts) - 1],
@@ -422,7 +426,7 @@ class CSVLocale(Locale):
 
 
 class GettextLocale(Locale):
-    """Locale implementation using the gettext module."""
+    """Locale implementation using the `gettext` module."""
     def __init__(self, code, translations):
         try:
             # python 2
@@ -444,66 +448,66 @@ class GettextLocale(Locale):
             return self.gettext(message)
 
 LOCALE_NAMES = {
-    "af_ZA": {"name_en": u"Afrikaans", "name": u"Afrikaans"},
-    "am_ET": {"name_en": u"Amharic", "name": u'\u12a0\u121b\u122d\u129b'},
-    "ar_AR": {"name_en": u"Arabic", "name": u"\u0627\u0644\u0639\u0631\u0628\u064a\u0629"},
-    "bg_BG": {"name_en": u"Bulgarian", "name": u"\u0411\u044a\u043b\u0433\u0430\u0440\u0441\u043a\u0438"},
-    "bn_IN": {"name_en": u"Bengali", "name": u"\u09ac\u09be\u0982\u09b2\u09be"},
-    "bs_BA": {"name_en": u"Bosnian", "name": u"Bosanski"},
-    "ca_ES": {"name_en": u"Catalan", "name": u"Catal\xe0"},
-    "cs_CZ": {"name_en": u"Czech", "name": u"\u010ce\u0161tina"},
-    "cy_GB": {"name_en": u"Welsh", "name": u"Cymraeg"},
-    "da_DK": {"name_en": u"Danish", "name": u"Dansk"},
-    "de_DE": {"name_en": u"German", "name": u"Deutsch"},
-    "el_GR": {"name_en": u"Greek", "name": u"\u0395\u03bb\u03bb\u03b7\u03bd\u03b9\u03ba\u03ac"},
-    "en_GB": {"name_en": u"English (UK)", "name": u"English (UK)"},
-    "en_US": {"name_en": u"English (US)", "name": u"English (US)"},
-    "es_ES": {"name_en": u"Spanish (Spain)", "name": u"Espa\xf1ol (Espa\xf1a)"},
-    "es_LA": {"name_en": u"Spanish", "name": u"Espa\xf1ol"},
-    "et_EE": {"name_en": u"Estonian", "name": u"Eesti"},
-    "eu_ES": {"name_en": u"Basque", "name": u"Euskara"},
-    "fa_IR": {"name_en": u"Persian", "name": u"\u0641\u0627\u0631\u0633\u06cc"},
-    "fi_FI": {"name_en": u"Finnish", "name": u"Suomi"},
-    "fr_CA": {"name_en": u"French (Canada)", "name": u"Fran\xe7ais (Canada)"},
-    "fr_FR": {"name_en": u"French", "name": u"Fran\xe7ais"},
-    "ga_IE": {"name_en": u"Irish", "name": u"Gaeilge"},
-    "gl_ES": {"name_en": u"Galician", "name": u"Galego"},
-    "he_IL": {"name_en": u"Hebrew", "name": u"\u05e2\u05d1\u05e8\u05d9\u05ea"},
-    "hi_IN": {"name_en": u"Hindi", "name": u"\u0939\u093f\u0928\u094d\u0926\u0940"},
-    "hr_HR": {"name_en": u"Croatian", "name": u"Hrvatski"},
-    "hu_HU": {"name_en": u"Hungarian", "name": u"Magyar"},
-    "id_ID": {"name_en": u"Indonesian", "name": u"Bahasa Indonesia"},
-    "is_IS": {"name_en": u"Icelandic", "name": u"\xcdslenska"},
-    "it_IT": {"name_en": u"Italian", "name": u"Italiano"},
-    "ja_JP": {"name_en": u"Japanese", "name": u"\u65e5\u672c\u8a9e"},
-    "ko_KR": {"name_en": u"Korean", "name": u"\ud55c\uad6d\uc5b4"},
-    "lt_LT": {"name_en": u"Lithuanian", "name": u"Lietuvi\u0173"},
-    "lv_LV": {"name_en": u"Latvian", "name": u"Latvie\u0161u"},
-    "mk_MK": {"name_en": u"Macedonian", "name": u"\u041c\u0430\u043a\u0435\u0434\u043e\u043d\u0441\u043a\u0438"},
-    "ml_IN": {"name_en": u"Malayalam", "name": u"\u0d2e\u0d32\u0d2f\u0d3e\u0d33\u0d02"},
-    "ms_MY": {"name_en": u"Malay", "name": u"Bahasa Melayu"},
-    "nb_NO": {"name_en": u"Norwegian (bokmal)", "name": u"Norsk (bokm\xe5l)"},
-    "nl_NL": {"name_en": u"Dutch", "name": u"Nederlands"},
-    "nn_NO": {"name_en": u"Norwegian (nynorsk)", "name": u"Norsk (nynorsk)"},
-    "pa_IN": {"name_en": u"Punjabi", "name": u"\u0a2a\u0a70\u0a1c\u0a3e\u0a2c\u0a40"},
-    "pl_PL": {"name_en": u"Polish", "name": u"Polski"},
-    "pt_BR": {"name_en": u"Portuguese (Brazil)", "name": u"Portugu\xeas (Brasil)"},
-    "pt_PT": {"name_en": u"Portuguese (Portugal)", "name": u"Portugu\xeas (Portugal)"},
-    "ro_RO": {"name_en": u"Romanian", "name": u"Rom\xe2n\u0103"},
-    "ru_RU": {"name_en": u"Russian", "name": u"\u0420\u0443\u0441\u0441\u043a\u0438\u0439"},
-    "sk_SK": {"name_en": u"Slovak", "name": u"Sloven\u010dina"},
-    "sl_SI": {"name_en": u"Slovenian", "name": u"Sloven\u0161\u010dina"},
-    "sq_AL": {"name_en": u"Albanian", "name": u"Shqip"},
-    "sr_RS": {"name_en": u"Serbian", "name": u"\u0421\u0440\u043f\u0441\u043a\u0438"},
-    "sv_SE": {"name_en": u"Swedish", "name": u"Svenska"},
-    "sw_KE": {"name_en": u"Swahili", "name": u"Kiswahili"},
-    "ta_IN": {"name_en": u"Tamil", "name": u"\u0ba4\u0bae\u0bbf\u0bb4\u0bcd"},
-    "te_IN": {"name_en": u"Telugu", "name": u"\u0c24\u0c46\u0c32\u0c41\u0c17\u0c41"},
-    "th_TH": {"name_en": u"Thai", "name": u"\u0e20\u0e32\u0e29\u0e32\u0e44\u0e17\u0e22"},
-    "tl_PH": {"name_en": u"Filipino", "name": u"Filipino"},
-    "tr_TR": {"name_en": u"Turkish", "name": u"T\xfcrk\xe7e"},
-    "uk_UA": {"name_en": u"Ukraini ", "name": u"\u0423\u043a\u0440\u0430\u0457\u043d\u0441\u044c\u043a\u0430"},
-    "vi_VN": {"name_en": u"Vietnamese", "name": u"Ti\u1ebfng Vi\u1ec7t"},
-    "zh_CN": {"name_en": u"Chinese (Simplified)", "name": u"\u4e2d\u6587(\u7b80\u4f53)"},
-    "zh_TW": {"name_en": u"Chinese (Traditional)", "name": u"\u4e2d\u6587(\u7e41\u9ad4)"},
+    "af_ZA": {"name_en": u("Afrikaans"), "name": u("Afrikaans")},
+    "am_ET": {"name_en": u("Amharic"), "name": u('\u12a0\u121b\u122d\u129b')},
+    "ar_AR": {"name_en": u("Arabic"), "name": u("\u0627\u0644\u0639\u0631\u0628\u064a\u0629")},
+    "bg_BG": {"name_en": u("Bulgarian"), "name": u("\u0411\u044a\u043b\u0433\u0430\u0440\u0441\u043a\u0438")},
+    "bn_IN": {"name_en": u("Bengali"), "name": u("\u09ac\u09be\u0982\u09b2\u09be")},
+    "bs_BA": {"name_en": u("Bosnian"), "name": u("Bosanski")},
+    "ca_ES": {"name_en": u("Catalan"), "name": u("Catal\xe0")},
+    "cs_CZ": {"name_en": u("Czech"), "name": u("\u010ce\u0161tina")},
+    "cy_GB": {"name_en": u("Welsh"), "name": u("Cymraeg")},
+    "da_DK": {"name_en": u("Danish"), "name": u("Dansk")},
+    "de_DE": {"name_en": u("German"), "name": u("Deutsch")},
+    "el_GR": {"name_en": u("Greek"), "name": u("\u0395\u03bb\u03bb\u03b7\u03bd\u03b9\u03ba\u03ac")},
+    "en_GB": {"name_en": u("English (UK)"), "name": u("English (UK)")},
+    "en_US": {"name_en": u("English (US)"), "name": u("English (US)")},
+    "es_ES": {"name_en": u("Spanish (Spain)"), "name": u("Espa\xf1ol (Espa\xf1a)")},
+    "es_LA": {"name_en": u("Spanish"), "name": u("Espa\xf1ol")},
+    "et_EE": {"name_en": u("Estonian"), "name": u("Eesti")},
+    "eu_ES": {"name_en": u("Basque"), "name": u("Euskara")},
+    "fa_IR": {"name_en": u("Persian"), "name": u("\u0641\u0627\u0631\u0633\u06cc")},
+    "fi_FI": {"name_en": u("Finnish"), "name": u("Suomi")},
+    "fr_CA": {"name_en": u("French (Canada)"), "name": u("Fran\xe7ais (Canada)")},
+    "fr_FR": {"name_en": u("French"), "name": u("Fran\xe7ais")},
+    "ga_IE": {"name_en": u("Irish"), "name": u("Gaeilge")},
+    "gl_ES": {"name_en": u("Galician"), "name": u("Galego")},
+    "he_IL": {"name_en": u("Hebrew"), "name": u("\u05e2\u05d1\u05e8\u05d9\u05ea")},
+    "hi_IN": {"name_en": u("Hindi"), "name": u("\u0939\u093f\u0928\u094d\u0926\u0940")},
+    "hr_HR": {"name_en": u("Croatian"), "name": u("Hrvatski")},
+    "hu_HU": {"name_en": u("Hungarian"), "name": u("Magyar")},
+    "id_ID": {"name_en": u("Indonesian"), "name": u("Bahasa Indonesia")},
+    "is_IS": {"name_en": u("Icelandic"), "name": u("\xcdslenska")},
+    "it_IT": {"name_en": u("Italian"), "name": u("Italiano")},
+    "ja_JP": {"name_en": u("Japanese"), "name": u("\u65e5\u672c\u8a9e")},
+    "ko_KR": {"name_en": u("Korean"), "name": u("\ud55c\uad6d\uc5b4")},
+    "lt_LT": {"name_en": u("Lithuanian"), "name": u("Lietuvi\u0173")},
+    "lv_LV": {"name_en": u("Latvian"), "name": u("Latvie\u0161u")},
+    "mk_MK": {"name_en": u("Macedonian"), "name": u("\u041c\u0430\u043a\u0435\u0434\u043e\u043d\u0441\u043a\u0438")},
+    "ml_IN": {"name_en": u("Malayalam"), "name": u("\u0d2e\u0d32\u0d2f\u0d3e\u0d33\u0d02")},
+    "ms_MY": {"name_en": u("Malay"), "name": u("Bahasa Melayu")},
+    "nb_NO": {"name_en": u("Norwegian (bokmal)"), "name": u("Norsk (bokm\xe5l)")},
+    "nl_NL": {"name_en": u("Dutch"), "name": u("Nederlands")},
+    "nn_NO": {"name_en": u("Norwegian (nynorsk)"), "name": u("Norsk (nynorsk)")},
+    "pa_IN": {"name_en": u("Punjabi"), "name": u("\u0a2a\u0a70\u0a1c\u0a3e\u0a2c\u0a40")},
+    "pl_PL": {"name_en": u("Polish"), "name": u("Polski")},
+    "pt_BR": {"name_en": u("Portuguese (Brazil)"), "name": u("Portugu\xeas (Brasil)")},
+    "pt_PT": {"name_en": u("Portuguese (Portugal)"), "name": u("Portugu\xeas (Portugal)")},
+    "ro_RO": {"name_en": u("Romanian"), "name": u("Rom\xe2n\u0103")},
+    "ru_RU": {"name_en": u("Russian"), "name": u("\u0420\u0443\u0441\u0441\u043a\u0438\u0439")},
+    "sk_SK": {"name_en": u("Slovak"), "name": u("Sloven\u010dina")},
+    "sl_SI": {"name_en": u("Slovenian"), "name": u("Sloven\u0161\u010dina")},
+    "sq_AL": {"name_en": u("Albanian"), "name": u("Shqip")},
+    "sr_RS": {"name_en": u("Serbian"), "name": u("\u0421\u0440\u043f\u0441\u043a\u0438")},
+    "sv_SE": {"name_en": u("Swedish"), "name": u("Svenska")},
+    "sw_KE": {"name_en": u("Swahili"), "name": u("Kiswahili")},
+    "ta_IN": {"name_en": u("Tamil"), "name": u("\u0ba4\u0bae\u0bbf\u0bb4\u0bcd")},
+    "te_IN": {"name_en": u("Telugu"), "name": u("\u0c24\u0c46\u0c32\u0c41\u0c17\u0c41")},
+    "th_TH": {"name_en": u("Thai"), "name": u("\u0e20\u0e32\u0e29\u0e32\u0e44\u0e17\u0e22")},
+    "tl_PH": {"name_en": u("Filipino"), "name": u("Filipino")},
+    "tr_TR": {"name_en": u("Turkish"), "name": u("T\xfcrk\xe7e")},
+    "uk_UA": {"name_en": u("Ukraini "), "name": u("\u0423\u043a\u0440\u0430\u0457\u043d\u0441\u044c\u043a\u0430")},
+    "vi_VN": {"name_en": u("Vietnamese"), "name": u("Ti\u1ebfng Vi\u1ec7t")},
+    "zh_CN": {"name_en": u("Chinese (Simplified)"), "name": u("\u4e2d\u6587(\u7b80\u4f53)")},
+    "zh_TW": {"name_en": u("Chinese (Traditional)"), "name": u("\u4e2d\u6587(\u7e41\u9ad4)")},
 }
